@@ -10,11 +10,11 @@ from .auto.auto import AutoExecutor as AutoExecutor
 from .core.core import Executor as Executor
 from .core.core import Job as Job
 from .core.job_environment import JobEnvironment as JobEnvironment
-from .local.debug import DebugExecutor as DebugExcecutor
+from .local.debug import DebugExecutor as DebugExecutor
 from .local.debug import DebugJob as DebugJob
 from .local.local import LocalExecutor as LocalExecutor
 from .local.local import LocalJob as LocalJob
 from .slurm.slurm import SlurmExecutor as SlurmExecutor
 from .slurm.slurm import SlurmJob as SlurmJob
 
-__version__ = "1.1.5"
+__version__ = "1.2.0"
