@@ -629,6 +629,37 @@ class Executor(abc.ABC):
             return []
         return self._internal_process_submissions(submissions)
 
+    def submit_array(self, fns: tp.Sequence[tp.Callable[[], R]]) -> tp.List[Job[R]]:
+        """Submit a list of job. This is useful when submiting different Checkpointable functions.
+        Be mindful that all those functions will be run with the same requirements
+        (cpus, gpus, timeout, ...). So try to make group of similar function calls.
+
+        Parameters
+        ----------
+        fns: list of callable
+            functions to compute. Those functions must not need any argument.
+            Tyically those are "Checkpointable" instance whose arguments
+            have been specified in the constructor, or partial functions.
+
+        Returns
+        -------
+        List[Job]
+            A list of Job instances.
+
+        Example
+        -------
+        a_vals = [1, 2, 3]
+        b_vals = [10, 20, 30]
+        fns = [functools.partial(int.__add__, a, b) for (a, b) in zip (a_vals, b_vals)]
+        executor.submit_array(fns)
+        # jobs will compute 1 + 10, 2 + 20, 3 + 30
+        """
+        submissions = [utils.DelayedSubmission(fn) for fn in fns]
+        if len(submissions) == 0:
+            warnings.warn("Received an empty job array")
+            return []
+        return self._internal_process_submissions(submissions)
+
     def update_parameters(self, **kwargs: tp.Any) -> None:
         """Update submision parameters."""
         if self._delayed_batch is not None:
@@ -707,12 +738,14 @@ class PicklingExecutor(Executor):
             A Job instance, providing access to the job information,
             including the output of the function once it is computed.
         """
+        eq_dict = self._equivalence_dict()
+        timeout_min = self.parameters.get(eq_dict["timeout_min"] if eq_dict else "timeout_min", 5)
         jobs = []
         for delayed in delayed_submissions:
             tmp_uuid = uuid.uuid4().hex
             pickle_path = utils.JobPaths.get_first_id_independent_folder(self.folder) / f"{tmp_uuid}.pkl"
             pickle_path.parent.mkdir(parents=True, exist_ok=True)
-            delayed.timeout_countdown = self.max_num_timeout
+            delayed.set_timeout(timeout_min, self.max_num_timeout)
             delayed.dump(pickle_path)
 
             self._throttle()
